package cn.wisenergy.chnmuseum.party.model;

import cn.wisenergy.chnmuseum.party.common.validator.groups.Add;
import cn.wisenergy.chnmuseum.party.common.validator.groups.Update;
import com.baomidou.mybatisplus.annotation.*;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.*;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.time.LocalDateTime;

/**
 * <p>
 * 文件资产临时
 * </p>
 *
 * @author Danny Lee
 * @since 2021-04-18
 */
@Data
@Builder
@AllArgsConstructor
@NoArgsConstructor
@Accessors(chain = true)
@EqualsAndHashCode(callSuper = false)
@TableName("asset_tmp")
@ApiModel(value = "文件资产临时", description = "文件资产临时")
public class AssetTmp implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty("文件资产ID")
    @TableId(value = "id", type = IdType.ASSIGN_ID)
    @NotNull(message = "文件资产ID不能为空", groups = {Add.class, Update.class})
    private String id;

    @ApiModelProperty("所属项ID")
    @TableField("ref_item_id")
    private String refItemId;

    @ApiModelProperty("原始文件名")
    @TableField("file_name")
    private String fileName;

    @ApiModelProperty("加密后文件名")
    @TableField("file_name_crypto")
    private String fileNameCrypto;

    @ApiModelProperty("扩展名")
    @TableField("file_ext_name")
    private String fileExtName;

    @ApiModelProperty("文件类型")
    @TableField("file_type")
    private String fileType;

    @ApiModelProperty("文件分类")
    @TableField("file_cat")
    private String fileCat;

    @ApiModelProperty("文件大小(B)")
    @TableField("file_size")
    private Long fileSize;

    @ApiModelProperty("下载链接")
    @TableField("file_url")
    private String fileUrl;

    @ApiModelProperty("加密链接")
    @TableField("file_url_crypto")
    private String fileUrlCrypto;

    @ApiModelProperty("视频缩略图")
    @TableField("thumbnail")
    private String thumbnail;

    @ApiModelProperty("语言")
    @TableField("language")
    private String language;

    @ApiModelProperty("MD5")
    @TableField("md5")
    private String md5;

    @TableField("crc32")
    private Long crc32;

    @TableField("video_content_name")
    private String videoContentName;

    @ApiModelProperty("创建日期")
    @TableField(value = "create_time", fill = FieldFill.INSERT)
    private LocalDateTime createTime;

    @ApiModelProperty("修改日期")
    @TableField(value = "update_time", fill = FieldFill.INSERT_UPDATE)
    private LocalDateTime updateTime;

}
